package com.loafle.overflow.central.module.discovery.service;

import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.central.commons.utils.SessionMetadata;
import com.loafle.overflow.core.annotation.ProbeAPI;
import com.loafle.overflow.core.annotation.WebappAPI;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.discovery.*;

import com.loafle.overflow.service.central.discovery.DiscoveryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * Created by snoop on 17. 9. 28.
 */
@Service("DiscoveryService")
public class CentralDiscoveryService implements DiscoveryService {

	@Autowired
	private MessagePublisher messagePublisher;

	@WebappAPI
	public void discoverZone(String probeID, DiscoverZone discoveryZone) throws OverflowException {
		String requesterSessionID = SessionMetadata.getSessionID();

		messagePublisher.publishToProbe(probeID, "DiscoveryService.DiscoverZone", requesterSessionID, discoveryZone);
	}

	@WebappAPI
	public void discoverHost(String probeID, Zone zone, DiscoverHost discoveryHost) throws OverflowException {
		String requesterSessionID = SessionMetadata.getSessionID();

		messagePublisher.publishToProbe(probeID, "DiscoveryService.DiscoverHost", requesterSessionID, zone, discoveryHost);
	}

	@WebappAPI
	public void discoverPort(String probeID, Host host, DiscoverPort discoveryPort) throws OverflowException {
		String requesterSessionID = SessionMetadata.getSessionID();

		messagePublisher.publishToProbe(probeID, "DiscoveryService.DiscoverPort", requesterSessionID, host, discoveryPort);
	}

	@WebappAPI
	public void discoverService(String probeID, Port port,
			com.loafle.overflow.model.discovery.DiscoverService discoveryService) throws OverflowException {
		String requesterSessionID = SessionMetadata.getSessionID();

		messagePublisher.publishToProbe(probeID, "DiscoveryService.DiscoverService", requesterSessionID, port,
				discoveryService);
	}

	@WebappAPI
	public void stopDiscovery(String probeID) throws OverflowException {
		String requesterSessionID = SessionMetadata.getSessionID();

		messagePublisher.publishToProbe(probeID, "DiscoveryService.StopDiscovery", requesterSessionID);
	}

	@ProbeAPI
	public void discoveryStart(String requesterSessionID, Date startDate) throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveryStart", startDate);
	}

	@ProbeAPI
	public void discoveryStop(String requesterSessionID, Date stopDate) throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveryStop", stopDate);
	}

	@ProbeAPI
	public void discoveredZone(String requesterSessionID, Zone zone) throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveredZone", zone);
	}

	@ProbeAPI
	public void discoveredHost(String requesterSessionID, Host host) throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveredHost", host);
	}

	@ProbeAPI
	public void discoveredPort(String requesterSessionID, Port port) throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveredPort", port);
	}

	@ProbeAPI
	public void discoveredService(String requesterSessionID, com.loafle.overflow.model.discovery.Service service)
			throws OverflowException {
		messagePublisher.publishToMemberSession(requesterSessionID, "DiscoveryService.discoveredService", service);
	}

}