package com.loafle.overflow.central.module.generator.service;

import com.loafle.overflow.central.commons.utils.StringConvertor;
import com.loafle.overflow.core.type.PortType;
import com.loafle.overflow.model.auth.AuthCrawler;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraService;
import com.loafle.overflow.model.meta.MetaSensorItemKey;
import com.loafle.overflow.model.meta.type.MetaCrawlerEnum;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.model.sensorconfig.Connection;
import com.loafle.overflow.model.sensorconfig.Crawler;
import com.loafle.overflow.model.sensorconfig.Schedule;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.loafle.overflow.model.sensorconfig.Target;
import com.loafle.overflow.service.central.auth.AuthCrawlerService;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("InfraServiceGenerator")
public class InfraServiceGenerator {

    @Autowired
    private GenerateUtil generateUtil;

    @Autowired
    private InfraServiceMysqlGenerator infraServiceMysqlGenerator;

    @Autowired
    private InfraServiceJMXGenerator infraServiceJMXGenerator;

    @Autowired
    private AuthCrawlerService authCrawlerService;

    @Autowired
    private ObjectMapper objectMapper;

    public String process(Sensor dbSensor, List<SensorItem> sensorItems, Infra infra) throws Exception {
        com.loafle.overflow.model.infra.InfraService infraService = (com.loafle.overflow.model.infra.InfraService)infra;

        SensorConfig sensorConfig = new SensorConfig();
        sensorConfig.setConfigID(String.valueOf(dbSensor.getId()));

        Target target = this.createTarget(infraService, dbSensor);

        if(target == null) {
            return null;
        }


        sensorConfig.setTarget(target);

        // FIXME: Interval
        Schedule schedule = new Schedule();
        schedule.setInterval("5");
        sensorConfig.setSchedule(schedule);

        Crawler crawler = this.generateUtil.getCrawler(dbSensor.getMetaCrawler());
        sensorConfig.setCrawler(crawler);

        Map<Integer, MetaSensorItemKey> keyMap = this.generateUtil.initMappingMap(dbSensor.getMetaCrawler());

        if(dbSensor.getMetaCrawler().getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
            this.infraServiceMysqlGenerator.process(sensorItems, keyMap, dbSensor, sensorConfig);
        } else if (dbSensor.getMetaCrawler().getId() == MetaCrawlerEnum.JMX_CRAWLER.getValue()) {
            this.infraServiceJMXGenerator.process(sensorItems, keyMap, dbSensor, sensorConfig);
        }

        return objectMapper.writeValueAsString(sensorConfig);
    }

    private Target createTarget(InfraService infraService, Sensor sensor) throws Exception {

        AuthCrawler authCrawler = this.authCrawlerService.readByMetaCrawlerIDAndTargetID(sensor.getMetaCrawler().getId(), sensor.getTarget().getId());

        if(authCrawler == null) {
            return null;
        }

        Target target = new Target();
        Connection connection = new Connection();
        connection.setIp(infraService.getInfraHost().getIpv4());
        connection.setPort(infraService.getPort());
        connection.setPortType(PortType.valueOf(infraService.getPortType()));
        connection.setSsl(infraService.isTlsType());

        target.setConnection(connection);

        HashMap<String, String> optionMap = this.objectMapper.readValue(authCrawler.getAuthJson(), new TypeReference<Map<String, String>>(){});

        Map<String, Object> auth = new HashMap<>();

        if(sensor.getMetaCrawler().getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
            auth.put("url", "jdbc:mysql://"+ infraService.getInfraHost().getIpv4() +":" + infraService.getPort());
            auth.put("id", optionMap.get("ID")); // FIXME: Auth Info
            auth.put("pw", optionMap.get("PassWord")); // FIXME: Auth Info
        } else if (sensor.getMetaCrawler().getId() == MetaCrawlerEnum.JMX_CRAWLER.getValue()) {
            auth.put("id", optionMap.get("ID")); // FIXME: Auth Info
            auth.put("pw", optionMap.get("PassWord")); // FIXME: Auth Info
            connection.setPort(9840);
        }

        target.setAuth(auth);

        return target;
    }

}
