package com.loafle.overflow.central.module.generator.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.List;

import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.sensor.SensorItemService;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("SensorConfigGenerator")
public class SensorConfigGenerator {

    @Autowired
    private SensorItemService sensorItemService;

    @Autowired
    private InfraService infraService;

    @Autowired
    private InfraHostGenerator infraHostGenerator;

    @Autowired
    private InfraServiceGenerator infraServiceGenerator;


    public String generate(Sensor sensor) throws Exception {
        PageParams pageParams = new PageParams();
        pageParams.setPageNo(0);
        pageParams.setCountPerPage(Integer.MAX_VALUE);
        pageParams.setSortCol("id");
        pageParams.setSortDirection("descending");
        Page<SensorItem> dbItemList = this.sensorItemService.readAllBySensorID(sensor.getId(), pageParams);

        List<SensorItem> sensorItems = dbItemList.getContent();

        if(sensorItems.size() <= 0) {
            return "";
        }
        Sensor dbSensor = sensorItems.get(0).getSensor();

        Infra infra = this.infraService.readByTargetID(dbSensor.getTarget().getId());

        // 7 = Infra OS Service
        if(infra.getMetaInfraType().getId() == 7) {
            return this.infraServiceGenerator.process(dbSensor, sensorItems, infra);
        }
        if(infra.getMetaInfraType().getId() == 2) {
            return this.infraHostGenerator.process(dbSensor, sensorItems, infra);
        }

        return null;
    }



}
