package com.loafle.overflow.central.module.infra.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.infra.dao.InfraDAO;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.probe.ProbeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */

@Service("InfraService")
public class CentralInfraService implements InfraService {

    @Autowired
    InfraDAO infraDAO;

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    public Infra regist(Infra infra) throws OverflowException {
        return this.infraDAO.save(infra);
    }

    public Infra read(Long id) throws OverflowException {
        Infra infra = this.infraDAO.findOne(id);
        return infra;
    }

    public Page<Infra> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
        return this.infraDAO.findAllByProbeId(probeID, PageUtil.getPageRequest(pageParams));
    }

    public Page<Infra> readAllByDomainID(Long domainID, PageParams pageParams) throws OverflowException {
        List<Probe> probeList = this.probeService.readAllByDomainID(domainID);

        if (probeList == null || probeList.size() <= 0) {
            throw new OverflowException("ProbeNotFoundException", new Throwable());
        }

        Page<Infra> infraList = this.infraDAO.findAllByProbeInAndTargetNotNull(probeList, PageUtil.getPageRequest(pageParams));
        return infraList;
    }

    public List<Target> readAllTargetByDomainID(Long domainID) throws OverflowException {

        List<Probe> probes = this.probeService.readAllByDomainID(domainID);

        if (probes == null || probes.size() <= 0) {
            throw new OverflowException("ProbeNotFoundException", new Throwable());
        }

        return this.infraDAO.findAllTargetByProbeIn(probes);
    }

    public List<Target> readAllTargetByProbes(List<Probe> probes) throws OverflowException {
        return this.infraDAO.findAllTargetByProbeIn(probes);
        // return null;
    }

    public Infra readByTargetID(Long targetID) throws OverflowException {
        return this.infraDAO.findByTargetId(targetID);
    }

}
