package com.loafle.overflow.central.module.noauthprobe.service;

import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.core.annotation.ProbeAPI;
import com.loafle.overflow.core.annotation.WebappAPI;
import com.loafle.overflow.central.commons.utils.GenerateKey;
import com.loafle.overflow.central.commons.utils.SessionMetadata;
import com.loafle.overflow.central.module.noauthprobe.dao.NoAuthProbeDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.apikey.ApiKey;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.domain.DomainMember;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.infra.InfraMachine;
import com.loafle.overflow.model.infra.InfraOS;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.meta.MetaInfraType;
import com.loafle.overflow.model.meta.MetaInfraVendor;
import com.loafle.overflow.model.meta.MetaNoAuthProbeStatus;
import com.loafle.overflow.model.meta.MetaProbeStatus;
import com.loafle.overflow.model.noauthprobe.NoAuthProbe;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescription;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescriptionHost;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescriptionNetwork;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.probe.ProbeHost;
import com.loafle.overflow.service.central.apikey.ApiKeyService;
import com.loafle.overflow.service.central.domain.DomainMemberService;
import com.loafle.overflow.service.central.infra.InfraHostService;
import com.loafle.overflow.service.central.infra.InfraMachineService;
import com.loafle.overflow.service.central.infra.InfraOSService;
import com.loafle.overflow.service.central.noauthprobe.NoAuthProbeService;
import com.loafle.overflow.service.central.probe.ProbeHostService;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.transaction.Transactional;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("NoAuthProbeService")
public class CentralNoAuthProbeService implements NoAuthProbeService {

  @Autowired
  private NoAuthProbeDAO noAuthProbeDAO;

  @Autowired
  private ApiKeyService apiKeyService;

  @Autowired
  private ProbeService probeService;

  @Autowired
  private MessagePublisher messagePublisher;

  @Autowired
  private DomainMemberService domainMemberService;

  @Autowired
  private ObjectMapper objectMapper;

  @Autowired
  private InfraMachineService infraMachineService;
  @Autowired
  private InfraOSService infraOSService;
  @Autowired
  private InfraHostService infraHostService;
  @Autowired
  private ProbeHostService probeHostService;

  @ProbeAPI
  public NoAuthProbe regist(NoAuthProbe noAuthProbe) throws OverflowException {

    ApiKey apiKey = apiKeyService.readByApiKey(noAuthProbe.getApiKey());
    noAuthProbe.setDomain(apiKey.getDomain());

    noAuthProbe.setTempProbeKey(GenerateKey.getKey());
    noAuthProbe.setMetaNoAuthProbeStatus(new MetaNoAuthProbeStatus((short) 3));

    messagePublisher.publishToDomainMembers(apiKey.getDomain().getId(), "NoAuthProbeService.regist", noAuthProbe);

    return this.noAuthProbeDAO.save(noAuthProbe);
  }

  public List<NoAuthProbe> readAllByDomainID(Long domainID) throws OverflowException {

    return this.noAuthProbeDAO.findAllByDomainIdAndMetaNoAuthProbeStatusId(domainID, (short) 3);
  }

  public NoAuthProbe read(Long id) {
    return this.noAuthProbeDAO.findOne(id);
  }

  @WebappAPI
  @Transactional
  public List<NoAuthProbe> acceptNoAuthProbe(Long noAuthProbeID) throws OverflowException {
    NoAuthProbe noAuthProbe = this.noAuthProbeDAO.findOne(noAuthProbeID);

    NoAuthProbeDescription noAuthProbeDescription = null;
    try {
      noAuthProbeDescription = this.objectMapper.readValue(noAuthProbe.getDescription(), NoAuthProbeDescription.class);
    } catch (IOException e) {
      throw new OverflowException("json error", e);
    }

    Probe probe = this.newProbe(noAuthProbe, noAuthProbeDescription);
    InfraMachine machine = this.newInfraMachine(noAuthProbe);
    InfraOS os = this.newInfraOS(machine, noAuthProbeDescription.getHost());
    InfraHost host = this.newInfraHost(os, noAuthProbeDescription.getNetwork());
    this.newProbeHost(host, probe);

    noAuthProbe.setMetaNoAuthProbeStatus(new MetaNoAuthProbeStatus((short) 1));
    this.noAuthProbeDAO.save(noAuthProbe);

    messagePublisher.publishToNoAuthProbe(noAuthProbe.getTempProbeKey(), "NoAuthProbeService.Accept",
        probe.getProbeKey());

    return this.readAllByDomainID(noAuthProbe.getDomain().getId());
  }

  private Probe newProbe(NoAuthProbe noauthprobe, NoAuthProbeDescription noAuthProbeDescription)
      throws OverflowException {

    BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
    String encryptKey = passwordEncoder.encode(UUID.randomUUID().toString());

    ApiKey apiKey = apiKeyService.readByApiKey(noauthprobe.getApiKey());
    String memberEmail = SessionMetadata.getTargetID();
    DomainMember domainMember = domainMemberService.readByMemberEmail(memberEmail);

    Probe probe = new Probe();
    probe.setEncryptionKey(encryptKey);
    probe.setProbeKey(GenerateKey.getKey());
    probe.setDomain(new Domain(apiKey.getDomain().getId()));
    probe.setAuthorizeMember(new Member(domainMember.getMember().getId()));
    probe.setTargetCount(0);
    probe.setMetaProbeStatus(new MetaProbeStatus((short) 1));

    String dispName = noAuthProbeDescription.getHost().getName();
    dispName += " probe";
    probe.setDisplayName(dispName);

    String addrStr = noAuthProbeDescription.getNetwork().getAddress();
    String[] addrArr = addrStr.split("\\|");
    probe.setCidr(addrArr[0]);

    return this.probeService.regist(probe);
  }

  private InfraMachine newInfraMachine(NoAuthProbe noauthprobe) throws OverflowException {
    MetaInfraType metaInfraType = new MetaInfraType(1);

    InfraMachine infraMachine = new InfraMachine();
    infraMachine.setMeta(noauthprobe.getDescription());
    infraMachine.setMetaInfraType(metaInfraType);

    return this.infraMachineService.regist(infraMachine);
  }

  private InfraOS newInfraOS(InfraMachine infraMachine, NoAuthProbeDescriptionHost noAuthProbeDescriptionHost)
      throws OverflowException {
    MetaInfraType metaInfraType = new MetaInfraType(3);

    InfraOS infraOS = new InfraOS();
    infraOS.setInfraMachine(infraMachine);
    infraOS.setMetaInfraType(metaInfraType);
    infraOS.setMetaInfraVendor(MetaInfraVendor.CreateInfraVendorByOS(noAuthProbeDescriptionHost.getOS()));

    return this.infraOSService.regist(infraOS);
  }

  private InfraHost newInfraHost(InfraOS infraOS, NoAuthProbeDescriptionNetwork noAuthProbeDescriptionNetwork)
      throws OverflowException {
    MetaInfraType metaInfraType = new MetaInfraType(2);

    InfraHost infraHost = new InfraHost();
    String addrStr = noAuthProbeDescriptionNetwork.getAddress();
    String[] addrArr = addrStr.split("\\|");

    infraHost.setIpv4(addrArr[0]);
    infraHost.setMac(noAuthProbeDescriptionNetwork.getMacAddress());
    infraHost.setInfraOS(infraOS);
    infraHost.setMetaInfraType(metaInfraType);

    return this.infraHostService.regist(infraHost);
  }

  private void newProbeHost(InfraHost infraHost, Probe probe) throws OverflowException {
    ProbeHost probeHost = new ProbeHost();
    probeHost.setHost(infraHost);
    probeHost.setProbe(probe);

    this.probeHostService.regist(probeHost);
  }

  @WebappAPI
  public List<NoAuthProbe> denyNoauthProbe(Long noAuthProbeID) throws OverflowException {
    NoAuthProbe noAuthProbe = this.noAuthProbeDAO.findOne(noAuthProbeID);

    noAuthProbe.setMetaNoAuthProbeStatus(new MetaNoAuthProbeStatus((short) 2));
    this.noAuthProbeDAO.save(noAuthProbe);

    messagePublisher.publishToNoAuthProbe(noAuthProbe.getTempProbeKey(), "NoAuthProbeService.Deny");

    return this.readAllByDomainID(noAuthProbe.getDomain().getId());
  }

  public NoAuthProbe readByTempProbeKey(String tempKey) throws OverflowException {
    return this.noAuthProbeDAO.findByTempProbeKey(tempKey);
  }

  public void onConnect(String tempKey, String connectAddress) throws OverflowException {
    NoAuthProbe noAuthProbe = this.noAuthProbeDAO.findByTempProbeKey(tempKey);
    noAuthProbe.setConnectDate(new Date());
    noAuthProbe.setConnectAddress(connectAddress);
    noAuthProbe = this.noAuthProbeDAO.save(noAuthProbe);

    messagePublisher.publishToDomainMembers(noAuthProbe.getDomain().getId(), "NoAuthProbeService.onConnect",
        noAuthProbe);
  }

  public void onDisconnect(String tempKey) throws OverflowException {
    NoAuthProbe noAuthProbe = this.noAuthProbeDAO.findByTempProbeKey(tempKey);
    noAuthProbe.setConnectDate(null);
    noAuthProbe.setConnectAddress(null);
    noAuthProbe = this.noAuthProbeDAO.save(noAuthProbe);

    messagePublisher.publishToDomainMembers(noAuthProbe.getDomain().getId(), "NoAuthProbeService.onDisconnect",
        noAuthProbe);
  }

}
