package com.loafle.overflow.central.module.notification.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.notification.dao.NotificationDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.notification.Notification;
import com.loafle.overflow.service.central.notification.NotificationService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by insanity on 17. 8. 25.
 */
@Service("NotificationService")
public class CentralNotificationService implements NotificationService {

    @Autowired
    private NotificationDAO notificationDAO;

    public Notification regist(Notification notification) throws OverflowException {
        return this.notificationDAO.save(notification);
    }

    public Page<Notification> readAllByMemberEmail(String memberEmail, PageParams pageParams) throws OverflowException {
        return this.notificationDAO.findAllByMemberEmail(memberEmail, PageUtil.getPageRequest(pageParams));
    }

    public Page<Notification> readAllUnconfirmedByMemberEmail(String memberEmail, PageParams pageParams)
            throws OverflowException {
        return this.notificationDAO.findAllByMemberEmailAndConfirmDateNull(memberEmail,
                PageUtil.getPageRequest(pageParams));
    }

    public Long readUnconfirmedCountByMemberEmail(String memberEmail) throws OverflowException {
        return this.notificationDAO.countByMemberEmailAndConfirmDateNull(memberEmail);
    }

    public Page<Notification> markAllAsReadByMemberEmail(String memberEmail, PageParams pageParams)
            throws OverflowException {
        List<Notification> list = this.notificationDAO.findAllByMemberEmail(memberEmail);
        for (Notification n : list) {
            n.setConfirmDate(new Date());
        }
        this.notificationDAO.save(list);
        return this.readAllByMemberEmail(memberEmail, pageParams);
    }

    public Page<Notification> markAllAsUnreadMemberEmail(String memberEmail, PageParams pageParams)
            throws OverflowException {
        List<Notification> list = this.notificationDAO.findAllByMemberEmail(memberEmail);
        for (Notification n : list) {
            n.setConfirmDate(null);
        }
        this.notificationDAO.save(list);
        return this.readAllByMemberEmail(memberEmail, pageParams);
    }

    public Notification markAsRead(Long notificationID) throws OverflowException {
        Notification notification = this.notificationDAO.findOne(notificationID);
        notification.setConfirmDate(new Date());
        return this.notificationDAO.save(notification);
    }

}
