package com.loafle.overflow.central.module.probe.service;

import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.central.module.probe.dao.ProbeDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("ProbeService")
public class CentralProbeService implements ProbeService {

    @Autowired
    private ProbeDAO probeDAO;
    @Autowired
    private MessagePublisher messagePublisher;

    public Probe regist(Probe probe) throws OverflowException {
        return this.probeDAO.save(probe);
    }

    public List<Probe> regist(List<Probe> probes) throws OverflowException {
        return this.probeDAO.save(probes);
    }

    public List<Probe> readAllByDomainID(Long domainID) throws OverflowException {
        List<Probe> probes = this.probeDAO.findAllByDomainIdOrderByIdDesc(domainID);
        return probes;
    }

    public Probe read(Long id) throws OverflowException {
        return this.probeDAO.findOne(id);
    }

    public Probe readByProbeKey(String probeKey) throws OverflowException {
        return this.probeDAO.findByProbeKey(probeKey);
    }

    public Probe modify(Probe probe) throws OverflowException {
        return this.probeDAO.save(probe);
    }

    public boolean remove(Long id) throws OverflowException {
        this.probeDAO.delete(id);
        return true;
    }

    public Probe increaseTargetCount(Probe probe) {
        Probe p = this.probeDAO.findOne(probe.getId());
        p.setTargetCount(p.getTargetCount() + 1);
        return this.probeDAO.save(p);
    }

    public Probe decreaseTargetCount(Probe probe) {
        Probe p = this.probeDAO.findOne(probe.getId());
        int count = p.getTargetCount();
        if (count > 0) {
            p.setTargetCount(count - 1);
        }
        return this.probeDAO.save(p);
    }

    public Probe modifyDisplayName(Long probeId, String displayName) {
        Probe probe = this.probeDAO.findOne(probeId);
        probe.setDisplayName(displayName);
        return this.probeDAO.save(probe);
    }

    public void onConnect(String probeKey, String connectAddress) throws OverflowException {
        Probe probe = this.probeDAO.findByProbeKey(probeKey);
        probe.setConnectDate(new Date());
        probe.setConnectAddress(connectAddress);
        probe = this.probeDAO.save(probe);

        messagePublisher.publishToDomainMembers(probe.getDomain().getId(), "ProbeService.onConnect", probe);
    }

    public void onDisconnect(String probeKey) throws OverflowException {
        Probe probe = this.probeDAO.findByProbeKey(probeKey);
        probe.setConnectDate(null);
        probe.setConnectAddress(null);
        probe = this.probeDAO.save(probe);

        messagePublisher.publishToDomainMembers(probe.getDomain().getId(), "ProbeService.onDisconnect", probe);
    }
}
