package com.loafle.overflow.central.module.sensor.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.generator.service.SensorConfigGenerator;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.meta.MetaSensorStatus;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.probe.ProbeService;
import com.loafle.overflow.service.central.sensor.SensorItemService;
import com.loafle.overflow.service.central.sensor.SensorService;
import com.loafle.overflow.service.central.target.TargetService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class CentralSensorService implements SensorService {

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private InfraService infraService;

    @Autowired
    private SensorItemService sensorItemService;

    @Autowired
    private SensorConfigGenerator sensorConfigGenerator;

    @Autowired
    private TargetService targetService;

    @Transactional
    public Sensor regist(Sensor sensor) throws OverflowException {
        this.targetService.increaseSensorCount(sensor.getTarget());
        return this.sensorDAO.save(sensor);
    }

    public Page<Sensor> readAllByDomainID(Long domainID, PageParams pageParams) throws OverflowException {

        List<Probe> probeList = this.probeService.readAllByDomainID(domainID);

        if (probeList == null || probeList.size() <= 0) {
            throw new OverflowException("", new Throwable());
        }

        List<Target> targetList = this.infraService.readAllTargetByProbes(probeList);

        if (targetList == null || targetList.size() <= 0) {
            throw new OverflowException("", new Throwable());
        }

        return this.sensorDAO.findAllByTargetIn(targetList, PageUtil.getPageRequest(pageParams));
    }

    public Page<Sensor> readAllByInfraID(Long infraID, PageParams pageParams) throws OverflowException {
        // Infra dbInfra = this.infraService.read(infraID);
        // if (dbInfra == null) {
        //     throw new OverflowException("", new Throwable());
        // }
        // return this.sensorDAO.findAllByTargetId(dbInfra.getTarget().getId(), PageUtil.getPageRequest(pageParams));
        return null;
    }

    public Page<Sensor> readAllByTargetID(Long targetID, PageParams pageParams) throws OverflowException {
        return this.sensorDAO.findAllByTargetId(targetID, PageUtil.getPageRequest(pageParams));
    }

    public Sensor read(Long id) throws OverflowException {
        return this.sensorDAO.findOne(Long.valueOf(id));
    }

    @Transactional
    public void remove(Long sensorID) throws OverflowException {
        Sensor sensor = this.sensorDAO.findOne(sensorID);
        this.targetService.decreaseSensorCount(sensor.getTarget());
        this.sensorDAO.delete(sensor);
    }

    public Sensor start(Long sensorID) throws OverflowException {
        Sensor sensor = this.sensorDAO.findOne(sensorID);
        MetaSensorStatus status = new MetaSensorStatus((short) 1);
        sensor.setMetaSensorStatus(status);
        return this.sensorDAO.save(sensor);
    }

    public Sensor stop(Long sensorID) throws OverflowException {
        Sensor sensor = this.sensorDAO.findOne(sensorID);
        MetaSensorStatus status = new MetaSensorStatus((short) 2);
        sensor.setMetaSensorStatus(status);
        return this.sensorDAO.save(sensor);
    }

    @Transactional
    public Sensor registSensorConfig(Sensor sensor, List<SensorItem> sensorItemList, String etcJson) throws OverflowException {

        this.targetService.increaseSensorCount(sensor.getTarget());
        this.sensorDAO.save(sensor);

        for (SensorItem sensorItem : sensorItemList) {
            sensorItem.setSensor(sensor);
        }

        this.sensorItemService.registAll(sensorItemList);

        return sensor;
    }

    public String generateSensorConfig(Sensor sensor) throws OverflowException {
        try {
			return this.sensorConfigGenerator.generate(sensor);
		} catch (Exception e) {
			throw new OverflowException("", new Throwable());
		}
    }

    //    public List<Sensor> readAllByTarget(Target target, PageParams pageParams) {
    //        return this.sensorDAO.findAllByTarget(target, PageUtil.getPageRequest(pageParams));
    //    }
}
